/**
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE":
 * The Cocoakit author wrote this file. As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return or, drink a beer in 
 * my honor.
 * ----------------------------------------------------------------------------
 */

#import "CKViewController.h"

#import "CKSwitchCell.h"

/**
 * Defines a NIB file name.
 */
#define NIB_FILE_NAME                                                           @"CKViewController"

@implementation CKViewController

#pragma mark -
#pragma mark Properties

@synthesize tableView = tableView_;

#pragma mark -
#pragma mark Memory management

/**
 * Deallocates the memory occupied by the receiver.
 */
- (void)dealloc {
    
    [tableView_ release];
    tableView_ = nil;
    
    [selectedCells_ release];
    selectedCells_ = nil;
    
    [super dealloc];
    
}

#pragma mark -
#pragma mark View lifecycle

/**
 * Prepares the receiver for service after it has been loaded from an Interface Builder archive, or nib file.
 */
- (void)awakeFromNib {
    
    [super awakeFromNib];
    
    selectedCells_ = [[NSMutableArray alloc] init];
    
}

/**
 * Called after the controller’s view is loaded into memory.
 */
- (void)viewDidLoad {
    
    [super viewDidLoad];
    
}

/**
 * Called when the controller’s view is released from memory.
 */
- (void)viewDidUnload {
    
    [super viewDidUnload];
    
    [self setTableView:nil];
    
}

#pragma mark -
#pragma mark Instances initialization

/*
 * Create and return an autoreleased CKViewController contructed from NIB file.
 */
+ (CKViewController *)ckViewController {
    
    CKViewController *result = [[[CKViewController alloc] initWithNibName:NIB_FILE_NAME bundle:nil] autorelease];
    
    [result awakeFromNib];
  
    return result;
    
}

#pragma mark -
#pragma mark UITableViewDataSource protocol selectors

/**
 * Tells the data source to return the number of rows in a given section of a table view. Returns the number of actions available,
 * three in this case.
 *
 * @param tableView: The table-view object requesting this information.
 * @param section: An index number identifying a section in tableView.
 * @return The number of actions available.
 */
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    
    return 20;
    
}

/**
 * Asks the data source for a cell to insert in a particular questions of the table view. A UITableViewCell is returned, containing
 * the action at the given question pressed.
 *
 * @param tableView: The table-view object requesting the cell.
 * @param indexPath: An index path locating a row in tableView.
 * @return An object inheriting from UITableViewCell that the table view can use for the specified row.
 */
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    CKSwitchCell *result = (CKSwitchCell *)[tableView dequeueReusableCellWithIdentifier:[CKSwitchCell cellIdentifier]];
    
    if (result == nil) {
        
        result = [CKSwitchCell ckSwitchCell];
        
        [result setDelegate:self];
        
        [result setIndex:[indexPath row]];
        
    }
    
    [[result textLabel] setText:[NSString stringWithFormat:@"CKSwitchCell: %d", [indexPath row]]];
    
    NSUInteger found = [selectedCells_ indexOfObject:[NSNumber numberWithUnsignedInteger:[result index]]];
    
    if (found < 20) {
        
        [[result switchView] setOn:YES];
        
    } else {
        
        [[result switchView] setOn:NO];
        
    }
    
    return result;
    
}

#pragma mark -
#pragma mark UITableViewDelegate protocol selectors

/**
 * Tells the delegate that the specified row is now selected.
 *
 * @param tableView: A table-view object informing the delegate about the new row selection.
 * @param indexPath: An index path locating the new selected row in tableView.
 */
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    [tableView deselectRowAtIndexPath:indexPath
                             animated:YES];
    
}

#pragma mark -
#pragma mark CKSwitchCellDelegate protocol selectors

/**
 * Class tell to delegate to indicate the selected switch.
 *
 * @param index: The index of cell.
 * @param isON: Flag to know if switch is ON or NO.
 */
- (void)didSelectedSwitchWithIndex:(NSUInteger)index enable:(BOOL)isON {
    
    if (isON) {
        
        [selectedCells_ addObject:[NSNumber numberWithUnsignedInteger:index]];
        
    } else {
        
        [selectedCells_ removeObject:[NSNumber numberWithUnsignedInteger:index]];
        
    }
    
}

#pragma mark - 
#pragma mark UINavigationBar

/**
 * The navigation item used to represent the view controller. (read-only)
 */
- (UINavigationItem *)navigationItem {
    
    UINavigationItem *result = [super navigationItem];
    
    [result setTitle:@"CKSwitchTableView"];
    
    return result;
    
}

@end
